﻿Imports ThinkGeo.MapSuite.Core
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.IO

Partial Class _Default
    Inherits System.Web.UI.Page
    Dim zipMapSourcePath As String = "..\..\Data\"
    Dim output As String = "c:\\newShapeFiles\\"

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
        If Not Page.IsPostBack Then
            Map1.MapBackground.BackgroundBrush = New GeoSolidBrush(GeoColor.FromHtml("#B3C6D4"))
            Map1.MapUnit = GeographyUnit.DecimalDegree
        End If
    End Sub

    Protected Sub btnCreatedCombinedZip_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCreatedCombinedZip.Click
        If Directory.Exists(output) Then
            Directory.Delete(output, True)
        End If

        Directory.CreateDirectory(output)

        Dim ZipRows1 As New Collection(Of ZipRow)
        ZipRows1.Add(New ZipRow("DE", 1))
        ZipRows1.Add(New ZipRow("DE", 2))
        CreateCombinedZips(ZipRows1)

        Dim ZipRows2 As New Collection(Of ZipRow)
        ZipRows2.Add(New ZipRow("DC", 1))
        ZipRows2.Add(New ZipRow("DC", 2))
        CreateCombinedZips(ZipRows2)

        ScriptManager.RegisterStartupScript(Page, Page.GetType(), "Message", "alert('2 shape files are created successfully under folder " + output + "');", True)
    End Sub

    Public Sub CreateCombinedZips(ByVal ZipRows As Collection(Of ZipRow))
        Dim zipShapes As New Collection(Of AreaBaseShape)

        Dim featureId As String = String.Empty
        For Each zipRow In ZipRows
            Dim zipLayer As New ShapeFileFeatureLayer(MapPath(String.Format("~/app_data/{0}/{0}zcta5cu.shp", zipRow.STATE)))
            zipLayer.RequireIndex = False
            zipLayer.Open()

            Dim zipFeature As Feature = zipLayer.FeatureSource.GetFeatureById(zipRow.GIST_ID.ToString, New String() {})
            zipShapes.Add(zipFeature.GetShape())
            featureId = zipRow.STATE & zipRow.GIST_ID
        Next

        ' Get combined Zip
        Dim combinedZips As MultipolygonShape = AreaBaseShape.Union(zipShapes)
        Dim dict As New Dictionary(Of String, String)
        dict.Add("FEATID", featureId)
        Dim feature As New Feature(combinedZips, dict)

        ' Get projected Zip
        Dim proj4 As New Proj4Projection(Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString())
        proj4.Open()
        Dim featureUnderGoogleMapProjection As Feature = proj4.ConvertToExternalProjection(feature)
        proj4.Close()

        ' Create Shape File
        Dim dbfColumns As New Collection(Of DbfColumn)
        dbfColumns.Add(New DbfColumn("FEATID", DbfColumnType.String, 6, 0))


        ShapeFileFeatureSource.CreateShapeFile(ShapeFileType.Polygon, output & featureId & ".shp", dbfColumns)
        ShapeFileFeatureLayer.BuildIndexFile(output & featureId & ".shp")

        ' Add Feature to the shape file
        Dim newShapeFile As New ShapeFileFeatureLayer(output & featureId & ".shp", ShapeFileReadWriteMode.ReadWrite)

        newShapeFile.Open()
        newShapeFile.EditTools.BeginTransaction()
        ' here you can decide you want the shape file in WGS84 or under Google projection
        Dim feature1 = featureUnderGoogleMapProjection 'feature
        newShapeFile.EditTools.Add(feature1)
        Dim results As TransactionResult = newShapeFile.EditTools.CommitTransaction()
        newShapeFile.Close()

    End Sub

    Private Sub DisplayMapWithNewLayers(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDisplayCombinedMap.Click
        Dim outputFilePaths As String() = Directory.GetFiles(output, "*.shp")
        Map1.StaticOverlay.Layers.Clear()

        For Each file As String In outputFilePaths
            Dim layer As New ShapeFileFeatureLayer(file)
            layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
            Map1.StaticOverlay.Layers.Add(layer)
        Next

        Map1.MapUnit = GeographyUnit.Meter
        Map1.CurrentExtent = GetBoundingBox(Map1.StaticOverlay.Layers)
    End Sub

    Private Function GetBoundingBox(ByVal layers As IEnumerable(Of Layer)) As RectangleShape

        Dim rectangleShape As New RectangleShape(0, 0, 0, 0)
        Dim minX As Double = Double.MaxValue
        Dim minY As Double = Double.MaxValue
        Dim maxX As Double = Double.MinValue
        Dim maxY As Double = Double.MinValue
        For Each layer As Layer In layers
            layer.Open()
            Dim boundingBox As RectangleShape = layer.GetBoundingBox()
            layer.Close()
            minX = If(boundingBox.LowerLeftPoint.X < minX, boundingBox.LowerLeftPoint.X, minX)
            minY = If(boundingBox.LowerLeftPoint.Y < minY, boundingBox.LowerLeftPoint.Y, minY)
            maxX = If(boundingBox.UpperRightPoint.X > maxX, boundingBox.UpperRightPoint.X, maxX)
            maxY = If(boundingBox.UpperRightPoint.Y > maxY, boundingBox.UpperRightPoint.Y, maxY)
        Next

        Return New RectangleShape(minX, maxY, maxX, minY)

    End Function
End Class

Public Structure ZipRow
    Public Sub New(ByVal state As String, ByVal gist_id As Integer)
        Me.STATE = state
        Me.GIST_ID = gist_id
    End Sub


    Dim STATE As String
    Dim GIST_ID As Integer
End Structure
